####################################
### LOAD PACKAGES
####################################

library(readxl)
library(robustbase)
library(broom)
library(sp)
library(plm)
library(dplyr)
library(maptools)
library(rgdal)
library(ggplot2)
library(RColorBrewer)
library(dichromat)
library(splm)
library(spdep)
library(dendextend)
library(spatialreg)
library(lmtest)
library(sandwich)

####################################
### DATA PREPARATION
####################################

#Set your working directory:
setwd("C://Users/TP/Desktop")

#Load the dataset:
mydata <- read_excel("Data.xlsx")

#Order the dataset in ascending order of PSC (needed later for the spatial weights matrix):
mydata <- mydata[order(mydata$psc),]

#Create a new variable for the total sales of accommodation facilities in the summer time:
mydata$ubytS <- mydata$jun55100trz + mydata$jul55100trz + mydata$aug55100trz + mydata$sep55100trz +
  mydata$jun55200trz + mydata$jul55200trz + mydata$aug55200trz + mydata$sep55200trz +
  mydata$jun55300trz + mydata$jul55300trz + mydata$aug55300trz + mydata$sep55300trz

#Create a new variable for the total sales of accommodation facilities in the winter time:
mydata$ubytW <- mydata$trzUZZ

#Create a new variable for the total municipal incomes:
mydata$prijm_rozp <- (mydata$prijm_bezn_rozp + mydata$prijm_kap_rozp) / mydata$obyvpocet

#Change some of the variables to logarithmic ones (sometimes log(1+x) to avoid having log0):
mydata$ubytSL <- log(1+mydata$ubytS)
mydata$ubytWL <- log(1+mydata$ubytW)

mydata$priem_st_ochranyL <- log(mydata$priem_st_ochrany)
mydata$najvyssi_vrchL <- log(mydata$najvyssi_vrch)

mydata$remoteL <- log(1+mydata$remote)
mydata$high_min_BL <- log(mydata$high_min_B)
mydata$high_min_2016L <- log(mydata$high_min_2016)

mydata$ceny_bytovL <- log(mydata$ceny_bytov)
mydata$prijm_rozpL <- log(1+mydata$prijm_rozp)

mydata$obyvpocetL <- log(mydata$obyvpocet)
mydata$rozloha_km2L <- log(mydata$rozloha_km2)
mydata$densityL <- log(mydata$density)

#Keep only the variables which are relevant for the analysis:
mydata <- subset(mydata, select = c(psc, ubytSL, ubytWL, priem_st_ochranyL,
                                    najvyssi_vrchL, remoteL, high_min_BL,
                                    high_min_2016L, nez_jun_2020, ceny_bytovL,
                                    prijm_rozpL, obyvpocetL, rozloha_km2L,
                                    densityL, mesto, lyz_str, zalesnenost_v_perc))

#Remove the missing values:
mydata <- na.omit(mydata)

####################################
### MODEL DIAGNOSTICS
####################################

#Get a correlation matrix:
res <- cor(mydata)
round(res, 2)

####################################
### ESTIMATION OF BASELINE MODEL
####################################

#Get a summary of the models for both summer and winter (with basic standard errors):

modelSL <- lm(ubytSL ~ priem_st_ochranyL + najvyssi_vrchL  + remoteL +
                nez_jun_2020 + ceny_bytovL + prijm_rozpL + obyvpocetL + rozloha_km2L + mesto,
              data = mydata)

summary(modelSL)

modelWL <- lm(ubytWL ~ priem_st_ochranyL + najvyssi_vrchL  + remoteL + lyz_str +
                nez_jun_2020 + ceny_bytovL + prijm_rozpL + obyvpocetL + rozloha_km2L + mesto,
              data = mydata)

summary(modelWL)

#Check the baseline model with "zalesnenost_v_perc" included in the model:
modelSLzal <- lm(ubytSL ~ priem_st_ochranyL + najvyssi_vrchL + zalesnenost_v_perc + remoteL +
              nez_jun_2020 + ceny_bytovL + prijm_rozpL + obyvpocetL + rozloha_km2L + mesto,
            data = mydata)

summary(modelSLzal)


modelWLzal <- lm(ubytWL ~ priem_st_ochranyL + najvyssi_vrchL + zalesnenost_v_perc + remoteL + lyz_str +
                   nez_jun_2020 + ceny_bytovL + prijm_rozpL + obyvpocetL + rozloha_km2L + mesto,
                 data = mydata)

summary(modelWLzal)

####################################
### FURTHER MODEL DIAGNOSTICS
####################################

#Proceed with Ramsey reset test for omitted variables. Run the test on the second and third powers:
resettest(modelSL, power = 2:3, type = "regressor", data = mydata)
resettest(modelWL, power = 2:3, type = "regressor", data = mydata)

#Breusch-Pagan test:
bptest(modelSL)
bptest(modelWL)

##########################################
### ESTIMATES WITH ROBUST STANDARD ERRORS 
##########################################

#Ask for a summary of the model:
#summary(mS)

#Get heteroskedaticity-robust standard errors for the model:
coeftest(modelSL, vcov = vcovHC(modelSL, type = "HC0"))
coeftest(modelWL, vcov = vcovHC(modelWL, type = "HC0"))

##################################
######### SPATIAL MODEL #########
#################################

#Prepare spatial weights matrix:
Rmatrix1 <- read.csv("CasovaMatica.csv")
Rmatrix2 <- Rmatrix1[,-1]
rownames(Rmatrix2) <- Rmatrix1[,1]
#Order the matrix in ascending order of PSC:
Rmatrix3 <- sort_dist_mat(Rmatrix2, by_rows = TRUE)
Rmatrix4 <- data.matrix(Rmatrix3, rownames.force = NA)
Rmatrix5 <- mat2listw(Rmatrix4, style="W") #Style "W" will create a row-standardized matrix: http://wlm.userweb.mwn.de/R/wlmRspma.htm
Rmatrix6 <- listw2mat(Rmatrix5)

# Construct spatially lagged variables:
mydata$priem_st_ochranySL <- lag.listw(Rmatrix5, mydata$priem_st_ochranyL)
mydata$najvyssi_vrchSL <- lag.listw(Rmatrix5, mydata$najvyssi_vrchL)
mydata$lyz_strSL <- lag.listw(Rmatrix5, mydata$lyz_str)
mydata$remoteSL <- lag.listw(Rmatrix5, mydata$remoteL)
mydata$ceny_bytovSL <- lag.listw(Rmatrix5, mydata$ceny_bytovL)
mydata$obyvpocetSL <- lag.listw(Rmatrix5, mydata$obyvpocetL)
mydata$mestoSL <- lag.listw(Rmatrix5, mydata$mesto)

##############################################################################
#Fit the following model which includes spatially lagged independent variables
##############################################################################

#For summer:
m1SL <- lm(ubytSL ~ priem_st_ochranyL + najvyssi_vrchL  + remoteL + nez_jun_2020 +
             ceny_bytovL + prijm_rozpL + obyvpocetL + rozloha_km2L + mesto +
             priem_st_ochranySL + najvyssi_vrchSL + remoteSL +
             ceny_bytovSL + obyvpocetSL + mestoSL,
           data = mydata)

#For winter:
m1WL <- lm(ubytWL ~ priem_st_ochranyL + najvyssi_vrchL + lyz_str + remoteL +
             nez_jun_2020 + ceny_bytovL + prijm_rozpL + obyvpocetL +
             rozloha_km2L + mesto + priem_st_ochranySL + najvyssi_vrchSL +
             lyz_strSL+ remoteSL + ceny_bytovSL + obyvpocetSL + mestoSL,
           data = mydata)

##########################################
### ESTIMATES SPATIAL MODELS 
##########################################

#SLX for summer:
summary(m1SL)

#SLX for winter:
summary(m1WL)

#Get heteroskedaticity-robust standard errors for the models above:
coeftest(m1SL, vcov = vcovHC(m1SL, type = "HC0"))
coeftest(m1WL, vcov = vcovHC(m1WL, type = "HC0"))

##########################################

#SEM (Spatial error model) for summer:
m1eS <- errorsarlm(modelSL, data=mydata, Rmatrix5, tol.solve=1.0e-30)
summary(m1eS)

#SEM (Spatial error model) for winter:
m1eW <- errorsarlm(modelWL, data=mydata, Rmatrix5, tol.solve=1.0e-30)
summary(m1eW)

##########################################

#SAR for summer:
m1sarS <- lagsarlm(modelSL,data=mydata, Rmatrix5, tol.solve=1.0e-30)
summary(m1sarS)

#SAR for winter:
m1sarW <- lagsarlm(modelWL,data=mydata, Rmatrix5, tol.solve=1.0e-30)
summary(m1sarW)

##########################################

#SDM for summer:
m1sdmS <- lagsarlm(m1SL,data=mydata, Rmatrix5, tol.solve=1.0e-30)
summary(m1sdmS)

#SDM for winter:
m1sdmW <- lagsarlm(m1WL,data=mydata, Rmatrix5, tol.solve=1.0e-30)
summary(m1sdmW)

######################################################